/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_QT_API_ACCOUNT_H
#define U1_QT_API_ACCOUNT_H

#include <QtCore/QObject>
#include <QtCore/QVariantMap>

#include "ApiExport.h"

namespace UbuntuOne
{
    namespace Api
    {

        class AccountPrivate;
        class CouchDB;
        class Subscription;

        /**
          Account object as returned from the Ubuntu One REST API.
          This class is more like a container specific for Account data.
        */
        class U1_QT_EXPORT Account : public QObject
        {
            Q_OBJECT

            // No READ for couchdb and subscription, since types would conflict!
            Q_PROPERTY(QVariantMap couchdb READ QVariantMap() WRITE setCouchdb)
            Q_PROPERTY(QString couchdb_root READ couchdbRoot WRITE setCouchdbRoot)
            Q_PROPERTY(QString email READ email WRITE setEmail)
            Q_PROPERTY(QString first_name READ firstName WRITE setFirstName)
            Q_PROPERTY(qulonglong id READ id WRITE setId)
            Q_PROPERTY(QString last_name READ lastName WRITE setLastName)
            Q_PROPERTY(QString nickname READ nickname WRITE setNickname)
            Q_PROPERTY(QString openid READ openid WRITE setOpenid)
            Q_PROPERTY(QVariantMap subscription READ QVariantMap() WRITE setSubscription)
            Q_PROPERTY(QString username READ username WRITE setUsername)

        public:
            /**
              Constructor.

              @param parent Parenting object.
            */
            explicit Account(QObject *parent = 0);

            /**
              Destructor.
            */
            virtual ~Account();

            /**
              Pointer to the current couchdb information for this Account.

              @return Pointer to Couchdb object.
            */
            CouchDB *couchdb() const;

            /**
              Setter for couchdb(). Technically it is more of a filler,
              since it will recurse and fill the Subscription instance with data.

              @param couchdb Must be valid json.
            */
            void setCouchdb(const QVariantMap &couchdb);

            /**
              Root path of the cloud's CouchDB for this account.

              @return URL.
            */
            QString couchdbRoot() const;

            /**
              Setter for couchdbRoot().
            */
            void setCouchdbRoot(const QString &couchdbRoot);

            /**
              Email address associated with this account.

              @return Email address.
            */
            QString email() const;

            /**
              Setter for email().
            */
            void setEmail(const QString &email);

            /**
              First name of the owner of this account.

              @return First name.
            */
            QString firstName() const;

            /**
              Setter for firstName().
            */
            void setFirstName(const QString &firstName);

            /**
              Id of this account.

              @return Id.
            */
            qulonglong id() const;

            /**
              Setter for id().
            */
            void setId(const qulonglong &id);

            /**
              Last name of the owner of this account.

              @return Last name.
            */
            QString lastName() const;

            /**
              Setter for lastName().
            */
            void setLastName(const QString &lastName);

            /**
              The combined first and last name of this account's owner.

              @return Combined first and last name.
            */
            QString nickname() const;

            /**
              Setter for nickname().
            */
            void setNickname(const QString &nickname);

            /**
              The OpenID URL associated with this account.

              @return OpenID URL.
            */
            QString openid() const;

            /**
              Setter for openid().
            */
            void setOpenid(const QString &openid);

            /**
              Pointer to the currently active subscription of this account.

              @return Pointer to Subscription object.
            */
            Subscription *subscription() const;

            /**
              Setter for subscription(). Technically it is more of a filler,
              since it will recurse and fill the Subscription instance with data.

              @param subscription Must be valid json.
            */
            void setSubscription(const QVariantMap &subscription);

            /**
              Username of this account's owner.

              @return Username.
            */
            QString username() const;

            /**
              Setter for username().
            */
            void setUsername(const QString &username);

        protected:
            AccountPrivate *const d_ptr;

        private:
            Q_DISABLE_COPY(Account)
            Q_DECLARE_PRIVATE(Account)
        };

    } // namespace Api
} // namespace UbuntuOne

#endif // U1_QT_API_ACCOUNT_H
