/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Api.h"

#include <KWallet/Wallet>

#include <QtCore/QDebug>
#include <qjson/parser.h>
#include <qjson/qobjecthelper.h>
#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkReply>
#include <QtNetwork/QNetworkRequest>
#include <QtOAuth/QtOAuth>

#include "Account.h"
#include "ComUbuntuSsoInterface.h"
#include "Device.h"
#include "Quota.h"

namespace UbuntuOne
{
    namespace Api
    {

        typedef QList<UbuntuOne::Api::Device> DeviceList;

        class InterfacePrivate
        {
        public:
            Q_DECLARE_PUBLIC(Interface);

            InterfacePrivate(Interface *q) : q_ptr(q), ready(false) {}
            virtual ~InterfacePrivate() {}
            Interface *const q_ptr;

            bool ready;

            QOAuth::Interface *m_oauth;
            QByteArray m_token;
            QByteArray m_secret;

            /**
              Generic getter function, called by the specific getters.

              @param url URL to get data from.
              @param manager Network access manager to use for request.
            */
            void get(QString url, QNetworkAccessManager *manager);

            /**
              Parse a network reply.

              This will ensure that no HTTP error was encounted, convert
              the received Json data to a QVariant and ensure no errors were
              encountered there, and finally apply the QVariant data onto a
              QObject.

              Please note that nested objects are currently not supported,
              so if you need this (for example the Account class does), then
              you need to implement the appropriate magic in the specific parser
              methods.

              @param object Object to fill with data from reply.
              @param reply Network reply to work with.

              @return Parsed and ready to go QVariant as returned by parseReply().
            */
            QVariant parse(QObject *object, QNetworkReply *reply);

            /**
              Called by parse() and responsible for working on the reply as well
              as getitng Json data converted into a QVariant.

              For the conversion from Json to QVariant parseJson() is used.

              @param reply Network reply to work with.

              @return Parsed and ready to go QVariant as returned by parseJson().
            */
            QVariant parseReply(QNetworkReply *reply);

            /**
              Parse a Json reply, and return QVariant.

              Currently this function will terminate if a parsing error is
              encountered.

              @param json Raw json data to parse into a QVariant.

              @return Parsed and ready to go QVariant (can be empty).
            */
            static QVariant parseJson(const QByteArray &json);

            /**
              Read access token and access secret from KWAllet and
              return them in ready to go QHash.

              @return Access token and secret in a QHash.
            */
            static QHash<QByteArray, QByteArray> getTokenFromWallet();
        };


        void InterfacePrivate::get(QString url, QNetworkAccessManager *manager)
        {
            QByteArray header =
                m_oauth->createParametersString(url, QOAuth::GET, m_token, m_secret,
                                                QOAuth::HMAC_SHA1, QOAuth::ParamMap(),
                                                QOAuth::ParseForHeaderArguments);

            qDebug() << header;

            QNetworkRequest request = QNetworkRequest(QUrl(url));
            request.setRawHeader("Authorization", header);

            manager->get(request);
        }

        QVariant InterfacePrivate::parse(QObject *object, QNetworkReply *reply)
        {
            QVariant data = parseReply(reply);
            QJson::QObjectHelper::qvariant2qobject(data.toMap(), object);
            return data;
        }

        QVariant InterfacePrivate::parseReply(QNetworkReply *reply)
        {
            Q_Q(Interface);

            QVariant data = QVariant();

            int returnCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
            QByteArray rawData = reply->readAll();

            qDebug() << returnCode;
            qDebug() << rawData;

            // The manager was created in the getFoo() method on the heap, so we
            // need to get rid of it now that we have received a reply.
            reply->manager()->deleteLater();

            if (returnCode != QOAuth::NoError) {
                qDebug() << "Got error, emitting gotError()";
                Q_EMIT q->gotError(returnCode, rawData);

                return data;
            }

            data = parseJson(rawData);

            return data;
        }

        QVariant InterfacePrivate::parseJson(const QByteArray &json)
        {
            bool ok;
            QJson::Parser parser;
            QVariant data = parser.parse(json, &ok);

            if (!ok) {
                qCritical("%i - Error: %s", parser.errorLine(), parser.errorString().toLatin1().data());
                exit(1);
            } else {
                qDebug() << "json object successfully converted to:";
                qDebug() << data;
            }

            return data;
        }

        QHash<QByteArray, QByteArray> InterfacePrivate::getTokenFromWallet()
        {
            KWallet::Wallet *wallet =
                KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet(),
                                            0, KWallet::Wallet::Synchronous);

            QString token;

            if (wallet && wallet->isOpen()) {
                if (wallet->setFolder("ubuntuone")) {
                    const QString str("ubuntuone");
                    wallet->readPassword(str, token);
                }
            }
            qDebug() << "token:" << token;

            delete wallet;

            QList<QString> tokenList = token.split('&');
            QHash<QByteArray, QByteArray> tokenHash;
            QString key;

            int separatorIndex;
            foreach(const QString & value, tokenList) {
                separatorIndex = value.indexOf('=');
                key = value.left(separatorIndex);
                tokenHash.insert(key.toAscii(), value.right(value.length() - separatorIndex - 1).toAscii());
            }

            return tokenHash;
        }

////////////////////////////// INTERFACE ////////////////////////////////

        Interface::Interface(QObject *parent) :
            QObject(parent),
            d_ptr(new InterfacePrivate(this))
        {
            Q_D(Interface);

            d->m_oauth = new QOAuth::Interface(this);

            ComUbuntuSsoInterface *auth =
                new ComUbuntuSsoInterface("com.ubuntu.sso",
                        "/", QDBusConnection::sessionBus(),
                        this);

            connect(auth, SIGNAL(NewCredentials(QString,QString)),
                    this, SLOT(authReady()));
            connect(auth, SIGNAL(AuthorizationDenied()),
                    this, SIGNAL(authorizationDenied()));
            connect(auth, SIGNAL(NoCredentials()),
                    this, SIGNAL(noCredentials()));
            connect(auth, SIGNAL(OAuthError(QString)),
                    this, SIGNAL(oauthError(QString)));

            auth->maybe_login("https://one.ubuntu.com", "ubuntuone", true);

            qRegisterMetaType<UbuntuOne::Api::DeviceList>("UbuntuOne::Api::DeviceList");
            qRegisterMetaType<UbuntuOne::Api::Subscription *>("Subscription");
        }

        Interface::~Interface()
        {
            delete d_ptr;
        }

        void Interface::getAccount()
        {
            Q_D(Interface);

            QString url("https://one.ubuntu.com/api/account/");
            QNetworkAccessManager *manager = new QNetworkAccessManager(this);
            connect(manager, SIGNAL(finished(QNetworkReply *)),
                    this, SLOT(parseAccount(QNetworkReply *)));

            d->get(url, manager);
        }

        void Interface::getDevices()
        {
            Q_D(Interface);

            QString url("https://one.ubuntu.com/api/1.0/devices/");
            QNetworkAccessManager *manager = new QNetworkAccessManager(this);
            connect(manager, SIGNAL(finished(QNetworkReply *)),
                    this, SLOT(parseDevices(QNetworkReply *)));

            d->get(url, manager);
        }

        void Interface::getQuota()
        {
            Q_D(Interface);

            QString url("https://one.ubuntu.com/api/quota/");
            QNetworkAccessManager *manager = new QNetworkAccessManager(this);
            connect(manager, SIGNAL(finished(QNetworkReply *)),
                    this, SLOT(parseQuota(QNetworkReply *)));

            d->get(url, manager);
        }

        void Interface::parseAccount(QNetworkReply *reply)
        {
            Q_D(Interface);

            Account account;

            if (d->parse(&account, reply).isValid()) {
                emit got(account);
            }
        }

        void Interface::parseDevices(QNetworkReply *reply)
        {
            Q_D(Interface);

            DeviceList deviceList = DeviceList();
            QVariant jsonData = d->parseReply(reply);
            QList<QVariant> variantList = jsonData.toList();
            foreach(const QVariant & deviceVariant, variantList) {
                Device tmpDevice;
                QJson::QObjectHelper::qvariant2qobject(deviceVariant.toMap(), &tmpDevice);
                deviceList.push_back(tmpDevice);
            }

            emit got(deviceList);
        }

        void Interface::parseQuota(QNetworkReply *reply)
        {
            Q_D(Interface);

            Quota quota;

            if (d->parse(&quota, reply).isValid()) {
                emit got(quota);
            }
        }

        void Interface::authReady()
        {
            Q_D(Interface);
            QHash<QByteArray, QByteArray> tokenHash = d->getTokenFromWallet();

            d->m_token = tokenHash["oauth_token"];
            d->m_secret = tokenHash["oauth_token_secret"];

            d->m_oauth->setConsumerKey("ubuntuone");
            d->m_oauth->setConsumerSecret("hammertime");
            d->m_oauth->setRequestTimeout(0);

            d->ready = true;
            emit ready();
        }

    } // namespace Api
} // namespace UbuntuOne
