/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_QT_API_INTERFACE_H
#define U1_QT_API_INTERFACE_H

#include "ApiExport.h"

#include <QtCore/QObject>

class QNetworkAccessManager;
class QNetworkReply;

namespace QOAuth
{
    class Interface;
}

namespace UbuntuOne
{

    /**
      Ubuntu One API namespace.

      The Ubuntu One API namespace provides access to the Ubuntu One Web API.
      Using this API you can access various informations regarding the currently
      authenticated account.

      Amoung other things you can access the following information:
      - Account Details
        - Username
        - Mail
        - Subscription
          - Upgrade options
      - Devices that are authenticated
      - Quota of the account
    */
    namespace Api
    {

        class Account;
        class Device;
        class InterfacePrivate;
        class Quota;

        /**
          Qt interface for the Ubuntu One REST API.

          This class implements asynchronious REST API queries with the Ubuntu One
          servers. All queries are executed using OAuth, using tokens and secrets
          as found in KWallet.
        */
        class U1_QT_EXPORT Interface : public QObject
        {
            Q_OBJECT

        public:
            /**
              Constructor.

              @param parent Parenting object.
            */
            explicit Interface(QObject *parent = 0);

            /**
              Destructor.
            */
            virtual ~Interface();

            /**
              Query for account information.
            */
            void getAccount();

            /**
              Query for a list of devices associated with this account.
            */
            void getDevices();

            /**
              Query for the current quota data.
            */
            void getQuota();

        Q_SIGNALS:
            /**
              Emitted if a query could not be parsed properly.

              @param code HTTP return code.
              @param html Raw content of the HTTP response.
            */
            void gotError(int code, QString html);

            /**
              Emitted once an Account object was successfully created.

              @param account Account object with data from REST query.
            */
            void got(const UbuntuOne::Api::Account &account);

            /**
              Emitted once a Quota object was successfully created.

              @param quota Quota object with data from REST query.
            */
            void got(const UbuntuOne::Api::Quota &quota);

            /**
              Emitted once a DeviceList was successfully created.

              @param deviceList DeviceList with data from REST query.
            */
            void got(const QList<UbuntuOne::Api::Device> &deviceList);

            /**
              Emitted if the user denied authorization.
            */
            void authorizationDenied();

            /**
              Emitted if no credentials could be obtained, for technical reasons.
            */
            void noCredentials();

            /**
              Emitted if no credentials could be obtained from a new OAuth session
              with the Ubuntu One servers.

              @param message A more precise description of what has gone wrong.
            */
            void oauthError(QString message);

            /**
              Emitted once the interface is ready (got authentication).
            */
            void ready();

        private Q_SLOTS:
            /**
              Called when a network reply arrives, that was issued for
              account data.

              @param reply Network reply.
            */
            void parseAccount(QNetworkReply *reply);

            /**
              Called when a network reply arrives, that was issued for
              device data.

              @param reply Network reply.
            */
            void parseDevices(QNetworkReply *reply);

            /**
              Called when a network reply arrives, that was issued for
              quota data.

              @param reply Network reply.
            */
            void parseQuota(QNetworkReply *reply);

            /**
              Called when authentication tokens were acquired by the auth
              helper.
            */
            void authReady();

        protected:
            InterfacePrivate *const d_ptr;

        private:
            Q_DISABLE_COPY(Interface)
            Q_DECLARE_PRIVATE(Interface)
        };

    } // namespace Api
} // namespace UbuntuOne

#endif // API_INTERFACE
