/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_QT_API_DEVICE_H
#define U1_QT_API_DEVICE_H

#include <QObject>

#include "ApiExport.h"

namespace UbuntuOne
{
    namespace Api
    {

        class DevicePrivate;

        /**
          Device object as returned from the Ubuntu One REST API.
          This class is more like a container specific for Device data.

          A Device in this context refers to any form of device that is
          authenticated for use with Ubuntu One. Every device has a unique token
          by which it is identifiable. Together with information regarding what
          kind of device it is and a description (usually the hostname) this
          makes for a pretty unique combination.
          Using this information you could for example remove authentication
          of a given device.
        */
        class U1_QT_EXPORT Device : public QObject
        {
            Q_OBJECT

            Q_PROPERTY(QString description READ description WRITE setDescription)
            Q_PROPERTY(QString kind READ kind WRITE setKind)
            Q_PROPERTY(QString token READ token WRITE setToken)

        public:
            explicit Device(QObject *parent = 0);
            virtual ~Device();

            Device(const Device &other);

            /**
              Assigns the specified Device data to this object.
            */
            Device &operator=(const Device &other);

            /**
              The description of the device. Usually this is the hostname of
              the device, however it does not have to be.

              @return description of the device.
            */
            QString description() const;

            /**
              Set the description of the device.

              @param description new description for the device.
            */
            void setDescription(const QString &description);

            /**
              The kind of the device. This could for example be "cell".

              @return the type of the device.
            */
            QString kind() const;

            /**
              Set the kind of the device.

              @param kind the type of the device.
            */
            void setKind(const QString &kind);

            /**
              The OAuth token of the device.

              @return the oauth token this device uses to communicate with the
                      Ubuntu One servers
            */
            QString token() const;

            /**
              Set the OAuth token of the device.

              @param token the new token of the device.
            */
            void setToken(const QString &token);

        protected:
            DevicePrivate *const d_ptr;

        private:
            Q_DECLARE_PRIVATE(Device)
        };

    } // namespace Api
} // namespace UbuntuOne

#endif // API_DEVICE_H
