/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Subscription.h"

#include <QtCore/QPointer>

#include <qjson/qobjecthelper.h>

namespace UbuntuOne
{
    namespace Api
    {

//////////////////////////////// OBJECT ////////////////////////////////
        class SubscriptionObjectPrivate
        {
        public:
            virtual ~SubscriptionObjectPrivate() {}
            QString m_currency;
            QString m_description;
            qulonglong m_id;
            QString m_name;
            double m_price;
        };

        SubscriptionObject::SubscriptionObject(QObject *parent) :
            QObject(parent),
            d_ptr(new SubscriptionObjectPrivate)
        {
        }

        SubscriptionObject::SubscriptionObject(SubscriptionObjectPrivate &dd,
                                               QObject *parent) :
            QObject(parent),
            d_ptr(&dd)
        {
        }

        SubscriptionObject::~SubscriptionObject()
        {
            delete d_ptr;
        }

        QString SubscriptionObject::currency() const
        {
            Q_D(const SubscriptionObject);

            return d->m_currency;
        }

        void SubscriptionObject::setCurrency(const QString &currency)
        {
            Q_D(SubscriptionObject);

            d->m_currency = currency;
        }

        QString SubscriptionObject::description() const
        {
            Q_D(const SubscriptionObject);

            return d->m_description;
        }

        void SubscriptionObject::setDescription(const QString &description)
        {
            Q_D(SubscriptionObject);

            d->m_description = description;
        }

        qulonglong SubscriptionObject::id() const
        {
            Q_D(const SubscriptionObject);

            return d->m_id;
        }

        void SubscriptionObject::setId(const qulonglong &id)
        {
            Q_D(SubscriptionObject);

            d->m_id = id;
        }

        QString SubscriptionObject::name() const
        {
            Q_D(const SubscriptionObject);

            return d->m_name;
        }

        void SubscriptionObject::setName(const QString &name)
        {
            Q_D(SubscriptionObject);

            d->m_name = name;
        }

        double SubscriptionObject::price() const
        {
            Q_D(const SubscriptionObject);

            return d->m_price;
        }

        void SubscriptionObject::setPrice(const double &price)
        {
            Q_D(SubscriptionObject);

            d->m_price = price;
        }

//////////////////////////////// UPGRADE ///////////////////////////////
        class SubscriptionUpgradePrivate : public SubscriptionObjectPrivate
        {
        public:
            QString m_upgradeUrl;
        };

        SubscriptionUpgrade::SubscriptionUpgrade(QObject *parent) :
            SubscriptionObject(*new SubscriptionUpgradePrivate, parent)
        {
        }


        SubscriptionUpgrade::SubscriptionUpgrade(SubscriptionUpgradePrivate &dd,
                QObject *parent) :
            SubscriptionObject(dd, parent)
        {
        }

        SubscriptionUpgrade::~SubscriptionUpgrade()
        {
        }

        QString SubscriptionUpgrade::upgradeUrl() const
        {
            Q_D(const SubscriptionUpgrade);

            return d->m_upgradeUrl;
        }

        void SubscriptionUpgrade::setUpgradeUrl(const QString &upgradeUrl)
        {
            Q_D(SubscriptionUpgrade);

            d->m_upgradeUrl = upgradeUrl;
        }

///////////////////////////// SUBSCRIPTION /////////////////////////////
        class SubscriptionPrivate : public SubscriptionObjectPrivate
        {
        public:
            QString m_expires;
            bool m_paid;
            QString m_started;
            bool m_trial;
            bool m_upgradeAvailable;
            QPointer<SubscriptionUpgrade> m_upgradeOption;
        };

        Subscription::Subscription(QObject *parent) :
            SubscriptionObject(*new SubscriptionPrivate, parent)
        {
        }

        Subscription::Subscription(SubscriptionPrivate &dd,
                                   QObject *parent) :
            SubscriptionObject(dd, parent)
        {
        }

        Subscription::~Subscription()
        {
        }

        QString Subscription::expires() const
        {
            Q_D(const Subscription);

            return d->m_expires;
        }

        void Subscription::setExpires(const QString &expires)
        {
            Q_D(Subscription);

            d->m_expires = expires;
        }

        bool Subscription::paid() const
        {
            Q_D(const Subscription);

            return d->m_paid;
        }

        void Subscription::setPaid(const bool &paid)
        {
            Q_D(Subscription);

            d->m_paid = paid;
        }

        QString Subscription::started() const
        {
            Q_D(const Subscription);

            return d->m_started;
        }

        void Subscription::setStarted(const QString &started)
        {
            Q_D(Subscription);

            d->m_started = started;
        }

        bool Subscription::trial() const
        {
            Q_D(const Subscription);

            return d->m_trial;
        }

        void Subscription::setTrial(const bool &trial)
        {
            Q_D(Subscription);

            d->m_trial = trial;
        }

        bool Subscription::upgradeAvailable() const
        {
            Q_D(const Subscription);

            return d->m_upgradeAvailable;
        }

        void Subscription::setUpgradeAvailable(const bool &upgradeAvailable)
        {
            Q_D(Subscription);

            d->m_upgradeAvailable = upgradeAvailable;
        }

        SubscriptionUpgrade *Subscription::upgradeOption() const
        {
            Q_D(const Subscription);

            if (d->m_upgradeOption.isNull()) {
                return 0;
            }
            return d->m_upgradeOption.data();
        }

        void Subscription::setUpgradeOption(const QVariantMap &upgradeOption)
        {
            Q_D(Subscription);

            if (upgradeOption.empty()) {
                return;
            }

            if (d->m_upgradeOption) {
                delete d->m_upgradeOption.data();
            }

            d->m_upgradeOption = new SubscriptionUpgrade(this);

            QJson::QObjectHelper::qvariant2qobject(upgradeOption,
                                                   this->upgradeOption());
        }

    } // namespace Api
} // namespace UbuntuOne
