/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_QT_API_SUBSCRIPTION_H
#define U1_QT_API_SUBSCRIPTION_H

#include <QtCore/QVariantMap>

#include "ApiExport.h"

namespace UbuntuOne
{
    namespace Api
    {

//////////////////////////////// OBJECT ////////////////////////////////
        class SubscriptionObjectPrivate;

        /**
          SubscriptionObject object as returned from the Ubuntu One REST API.
          This class is more like a container specific for SubscriptionObject data.

          This class should not be used itself but the real use implementations
          of a SubscriptionObject (Subscription and SubscriptionUpgrade).
          It is but the core of those two classes and the general base of
          SubscriptionObjects in the Ubuntu One API context.
        */
        class U1_QT_EXPORT SubscriptionObject : public QObject
        {
            Q_OBJECT

            Q_PROPERTY(QString currency READ currency WRITE setCurrency)
            Q_PROPERTY(QString description READ description WRITE setDescription)
            Q_PROPERTY(qulonglong id READ id WRITE setId)
            Q_PROPERTY(QString name READ name WRITE setName)
            Q_PROPERTY(double price READ price WRITE setPrice)

        public:
            explicit SubscriptionObject(QObject *parent = 0);
            virtual ~SubscriptionObject();

            /**
              The currency the subscription is priced in.

              @return currency (for example USD).
            */
            QString currency() const;

            /**
              Sets the currency of the subscription.

              @param currency the currency (for example USD).
            */
            void setCurrency(const QString &currency);

            /**
              The description of the subscription. Please note that this should
              be different from the name and actually really describe what this
              type of subscription is all about.

              A possilbe description might be:
              "Super expensive subscription with no service whatsoever."

              @return the description string.
            */
            QString description() const;

            /**
              Sets the description of the subscription.

              @param description the description.
            */
            void setDescription(const QString &description);

            /**
              The id of the subscription type.

              @return the id.
            */
            qulonglong id() const;

            /**
              Sets the id of the subscription type.

              @param id the id.
            */
            void setId(const qulonglong &id);

            /**
              The name of the subscription.

              @return the name
            */
            QString name() const;

            /**
              Sets the name of the subscription.

              @param name the name.
            */
            void setName(const QString &name);

            /**
              Price of the subscription.

              @return the price.
            */
            double price() const;

            /**
              Sets the price of the subscription.

              @param price the price.
            */
            void setPrice(const double &price);

        protected:
            SubscriptionObjectPrivate *const d_ptr;
            SubscriptionObject(SubscriptionObjectPrivate &dd, QObject *parent);

        private:
            Q_DISABLE_COPY(SubscriptionObject)
            Q_DECLARE_PRIVATE(SubscriptionObject)
        };

//////////////////////////////// UPGRADE ///////////////////////////////
        class SubscriptionUpgradePrivate;

        /**
          SubscriptionUpgrade object as returned from the Ubuntu One REST API.
          This class is more like a container specific for SubscriptionUpgrade data.

          This class implements a SubscriptionObject but additionally offers an
          upgrade URL where the user can upgrade her current subscription to
          the offered one.
        */
        class U1_QT_EXPORT SubscriptionUpgrade : public SubscriptionObject
        {
            Q_OBJECT

            Q_PROPERTY(QString upgrade_url READ upgradeUrl WRITE setUpgradeUrl)

        public:
            explicit SubscriptionUpgrade(QObject *parent = 0);
            virtual ~SubscriptionUpgrade();

            /**
              The URL where the user can upgrade to the subscription.

              @return a URL.
            */
            QString upgradeUrl() const;

            /**
              Sets the URL where the user can upgrade.

              @param upgradeUrl the url.
            */
            void setUpgradeUrl(const QString &upgradeUrl);

        protected:
            SubscriptionUpgrade(SubscriptionUpgradePrivate &dd, QObject *parent);

        private:
            Q_DISABLE_COPY(SubscriptionUpgrade)
            Q_DECLARE_PRIVATE(SubscriptionUpgrade)
        };

///////////////////////////// SUBSCRIPTION /////////////////////////////
        class SubscriptionPrivate;

        /**
          Subscription object as returned from the Ubuntu One REST API.
          This class is more like a container specific for Subscription data.

          This class implements a SubscriptionObject but additionally provides
          information related to the currently active Subscription.
          For example a Subscription can be paied or not or be in trial stage.

          Please note that this class is only meant to present the currently
          active subscription. For subscription upgrades please refer to
          SubscriptionUpgrade.
        */
        class U1_QT_EXPORT Subscription : public SubscriptionObject
        {
            Q_OBJECT

            Q_PROPERTY(QString expires READ expires WRITE setExpires)
            Q_PROPERTY(bool paid READ paid WRITE setPaid)
            Q_PROPERTY(QString started READ started WRITE setStarted)
            Q_PROPERTY(bool trial READ trial WRITE setTrial)
            Q_PROPERTY(bool upgrade_available READ upgradeAvailable WRITE setUpgradeAvailable)
            Q_PROPERTY(SubscriptionUpgrade *upgrade_option READ upgradeOption)
            Q_PROPERTY(QVariantMap upgrade_option READ QVariantMap() WRITE setUpgradeOption)

        public:
            explicit Subscription(QObject *parent = 0);
            virtual ~Subscription();

            /**
              Returns when this subscription will expire.

              @return expiration data as string.
            */
            QString expires() const;

            /**
              Set the date when this subscription expires.

              @param expires the string information.
            */
            void setExpires(const QString &expires);

            /**
              Whether this subscription is paied.

              @return @c true if the subscription is paied.
              @return @c flase if the subscription is not paied yet.
            */
            bool paid() const;

            /**
              Set whether the subscription is paied.

              @param paid true if the subscription is paied, false if not.
            */
            void setPaid(const bool &paid);

            /**
              The date when the subscription started running.

              @return date string.
            */
            QString started() const;

            /**
              Set the date string when the subscription started.

              @param started date string.
            */
            void setStarted(const QString &started);

            /**
              Whether the subscription is in trial period.

              @return @c true if it is a trial.
              @return @c false if it is not a trial.
            */
            bool trial() const;

            /**
              Set whether the subscription is a trial.

              @param trial true if in trial period, false if not.
            */
            void setTrial(const bool &trial);

            /**
              Wether an upgrade is available.
              Only if this is true upgradeOption() will return anything meaningful.

              Most of the time it will make more sense to simply call upgradeOption()
              and check if it returned 0.

              @return @c true if an upgrade is available.
              @return @c false if no upgrades is available.
            */
            bool upgradeAvailable() const;

            /**
              Sets whether an upgrade for the subscription is available.

              @pram upgradeAvailable true if upgrade is available, false if not.
            */
            void setUpgradeAvailable(const bool &upgradeAvailable);

            /**
              Returns an upgrade option for the subscription.

              @return @c SubscriptionUpgrade if there is any.
              @return @c 0 if there is no upgrade available.
            */
            SubscriptionUpgrade *upgradeOption() const;

            /**
              Set an upgrade option for the subscription.

              @param a QVariantMap (QStringHash to be precise) of the upgrade
                     option, the map will automatically be marshalled to a
                     SubscriptionUpgrade.
            */
            void setUpgradeOption(const QVariantMap &upgradeOption);

        protected:
            Subscription(SubscriptionPrivate &dd, QObject *parent);

        private:
            Q_DISABLE_COPY(Subscription)
            Q_DECLARE_PRIVATE(Subscription)
        };

    } // namespace Api
} // namespace UbuntuOne

#endif // API_SUBSCRIPTION_H
