/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "FolderModel.h"

#include <QtCore/QStringBuilder>

#include "SyncDaemon.h"
#include "SyncDaemonFolders.h"

namespace UbuntuOne {
    FolderModel::FolderModel() :
            QFileSystemModel(),
            m_daemon(new SyncDaemon(this)),
            m_folders(new SyncDaemonFolders(this))
    {
        setFilter(QDir::AllDirs | QDir::NoDotAndDotDot);

        // TODO: make directories updatable on the fly and connect appropriate
        // signals of m_folders with an appropriate slot on this end
        setDirectories(m_folders->folderList());
    }

    Qt::ItemFlags FolderModel::flags(const QModelIndex &index) const
    {
        Qt::ItemFlags flags = QFileSystemModel::flags(index);
        const QString path = filePath( index );
        if ((path == m_daemon->rootDir()) ||
            (recursive() && ancestorChecked(path)) ||
            isForbiddenPath(path)) {
            flags ^= Qt::ItemIsEnabled; //disabled!
        }

        flags |= Qt::ItemIsUserCheckable;

        return flags;
    }

    QVariant FolderModel::data(const QModelIndex& index, int role) const
    {
        if(index.isValid() && index.column() == 0 && role == Qt::CheckStateRole)
        {
            const QString path = filePath(index);
            if (path == m_daemon->rootDir()) { // Chacked andd disabled it shall be.
                return Qt::Checked;
            }
            if(recursive() && ancestorChecked(path)) {
                return Qt::Checked; // Always set children of recursively checked parents to checked.
            }
            if(isForbiddenPath(path)) {
                return Qt::Unchecked; // Forbidden paths can never be checked.
            }
            if(!m_checked.contains(path) && descendantChecked(path)) {
                return Qt::PartiallyChecked;
            }
            return m_checked.contains(path) ? Qt::Checked : Qt::Unchecked;
        }
        return QFileSystemModel::data(index, role);
    }

    bool FolderModel::setData( const QModelIndex& index, const QVariant& value, int role )
    {
        if( index.isValid() && index.column() == 0 && role == Qt::CheckStateRole )
        {
            QString path = filePath( index );
            // store checked paths, remove unchecked paths
            if(value.toInt() == Qt::Checked) {
                m_checked.insert(path);
            } else {
                m_checked.remove(path);
            }

            const QModelIndex &parentIndex = parent(index);
            const int lastRow = rowCount(parentIndex);
            emit dataChanged(sibling( 0, 0, parentIndex), sibling( lastRow, 0, parentIndex ));
            return true;
        }
        return QFileSystemModel::setData( index, value, role );
    }

    void FolderModel::setDirectories( const QStringList &dirs )
    {
        m_checked.clear();
        foreach( const QString &dir, dirs )
        {
            m_checked.insert( dir );
        }
        m_checked.insert(m_daemon->rootDir());
    }

    QStringList FolderModel::directories() const
    {
        QStringList dirs = m_checked.toList();

        qSort(dirs.begin(), dirs.end());

        // We need to remove any children of selected items as
        // they are redundant when recursive mode is chosen
        foreach( const QString &dir, dirs )
        {
            if( ancestorChecked( dir ) ) {
                dirs.removeAll( dir );
            }
        }

        dirs.removeAll(m_daemon->rootDir());

        return dirs;
    }

    inline bool FolderModel::isForbiddenPath(const QString &path) const
    {
        // we need the trailing slash otherwise we could forbid "/dev-music" for example
        QString _path = path.endsWith(QLatin1Char('/')) ? path : path % QLatin1Char('/');
        return _path.startsWith(QLatin1String("/proc/")) ||
               _path.startsWith(QLatin1String("/dev/")) ||
               _path.startsWith(QLatin1String("/sys/"));
    }

    void FolderModel::save()
    {
        qDebug() << "save" << directories();
        m_folders->changeFolders(directories());
    }

    bool FolderModel::ancestorChecked(const QString &path) const
    {
        // we need the trailing slash otherwise sibling folders with one as the prefix of the other are seen as parent/child
        const QString _path = path.endsWith(QLatin1Char('/')) ? path : path % QLatin1Char('/');

        foreach( const QString &element, m_checked )
        {
            const QString _element = element.endsWith(QLatin1Char('/')) ? element : element % QLatin1Char('/');
            if( _path.startsWith( _element ) && _element != _path ) {
                return true;
            }
        }
        return false;
    }

    bool FolderModel::descendantChecked(const QString& path) const
    {
        // we need the trailing slash otherwise sibling folders with one as the prefix of the other are seen as parent/child
        const QString _path = path.endsWith(QLatin1Char('/')) ? path : path % QLatin1Char('/');

        foreach(const QString &element, m_checked)
        {
            const QString _element = element.endsWith(QLatin1Char('/')) ? element : element % QLatin1Char('/');
            if( _element.startsWith( _path ) && _element != _path ) {
                return true;
            }
        }
        return false;
    }
} // namespace UbuntuOne
