/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Share.h"

#include "QtDBus/QDBusMetaType"

#include "Types.h"

namespace UbuntuOne {

    class SharePrivate
    {
    public:
        QString name;
        QString otherName;
        QString otherVisibleName;
        Share::Access accessLevel;
        QString nodeId;
        QString volumeId;
        uint freeBytes;
        QString path;
        bool accepted;
        Share::Type type;
    };

    Share::Share(QObject *parent) :
            QObject(parent),
            d_ptr(new SharePrivate)
    {
    }

    Share::~Share()
    {
    }

    Share::Share(const Share &other) :
        QObject(other.parent()),
        d_ptr(new SharePrivate)
    {
        *this = other;
    }

    Share &Share::operator=(const Share &other)
    {
        if (this == &other) {
            return *this;
        }

        Q_D(Share);

        d->name = other.name();
        d->otherName = other.otherName();
        d->otherVisibleName = other.otherVisibleName();
        d->accessLevel = other.accessLevel();
        d->nodeId = other.nodeId();
        d->volumeId = other.volumeId();
        d->freeBytes = other.freeBytes();
        d->path = other.path();
        d->accepted = other.isAccepted();
        d->type = other.type();

        return *this;
    }

    QString Share::name() const
    {
        Q_D(const Share);
        return d->name;
    }

    QString Share::otherName() const
    {
        Q_D(const Share);
        return d->otherName;
    }

    QString Share::otherVisibleName() const
    {
        Q_D(const Share);
        return d->otherVisibleName;
    }

    Share::Access Share::accessLevel() const
    {
        Q_D(const Share);
        return d->accessLevel;
    }

    QString Share::nodeId() const
    {
        Q_D(const Share);
        return d->nodeId;
    }

    QString Share::volumeId() const
    {
        Q_D(const Share);
        return d->volumeId;
    }

    uint Share::freeBytes() const
    {
        Q_D(const Share);
        return d->freeBytes;
    }

    QString Share::path() const
    {
        Q_D(const Share);
        return d->path;
    }

    bool Share::isAccepted() const
    {
        Q_D(const Share);
        return d->accepted;
    }

    Share::Type Share::type() const
    {
        Q_D(const Share);
        return d->type;
    }

    Share::Access Share::stringToAccess(const QString &access)
    {
        if (access == QLatin1String("View")) {
            return Share::Read;
        }
        return Share::Write;
    }

    QString Share::typeToString(Share::Type type)
    {
        if (type == Share::Inbound) {
            return QString("Share");
        }
        return QString("Shared");
    }

    Share::Type Share::stringToType(const QString &type)
    {
        if (type == QLatin1String("Share")) {
            return Share::Inbound;
        }
        return Share::Outbound;
    }

    QString Share::accessToString(Share::Access access)
    {
        if (access == Share::Read) {
            return QString("View");
        }
        return QString("Modify");
    }

    void Share::registerMetaType()
    {
        qRegisterMetaType<UbuntuOne::Share>("UbuntuOne::Share");
        qDBusRegisterMetaType<UbuntuOne::Share>();
        qRegisterMetaType<UbuntuOne::ShareList>("UbuntuOne::ShareList");
        qDBusRegisterMetaType<UbuntuOne::ShareList>();
    }

    // TODO: find some way to loop over all elements and (de)serialize that way.
    const QDBusArgument &operator>>(const QDBusArgument &argument,
                                    UbuntuOne::Share &share)
    {
        QStringHash myhash;
        argument >> myhash;

        SharePrivate *const d = share.d_func();
        d->name = myhash["name"];
        d->otherName = myhash["other_name"];
        d->otherVisibleName = myhash["other_visible_name"];
        d->accessLevel = Share::stringToAccess(myhash["access_level"]);
        d->nodeId = myhash["node_id"];
        d->volumeId = myhash["volume_id"];
        d->freeBytes = myhash["free_bytes"].toUInt();
        d->path = myhash["path"];
        d->accepted = myhash["accepted"] == QLatin1String("True") ? true : false;
        d->type = Share::stringToType(myhash["type"]);

        return argument;
    }

    QDBusArgument &operator<<(QDBusArgument &argument, const UbuntuOne::Share &share)
    {
        Q_UNUSED(share);
        // TODO: Implement proper when needed.
        argument.beginMap(QVariant::String, QVariant::String);
        argument.endMap();
        return argument;
    }
} // namespace UbuntuOne
