/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_KDE_SHARE_H
#define U1_KDE_SHARE_H

#include <QtCore/QObject>
#include <QtDBus/QDBusArgument>

#include "Export.h"

namespace UbuntuOne {
    class SharePrivate;

    /**
      An Ubuntu One Share.
    */
    class U1_KDE_EXPORT Share : public QObject
    {
        Q_OBJECT

    public:
        /**
          Access types for shares.
        */
        enum Access
        {
            Read = 0, /**< Read-only access */
            Write = 1 /**< Write access (implies read access) */
        };

        enum Type
        {
            Inbound = 0, /**< Incoming share, someone shared with the user */
            Outbound =1, /**< Outgoing share, the user is sharing with someone */
        };

        Share(QObject *parent = 0);
        virtual ~Share();

        Share(const Share &other);
        Share &operator=(const Share &other);

        /**
          @return the name of the share
        */
        QString name() const;

        /**
          @return the username or email address of the other share party (person)
        */
        QString otherName() const;

        /**
          @return the full name of the other share party (person)
        */
        QString otherVisibleName() const;

        /**
          @return the access type of the share
        */
        Share::Access accessLevel() const;

        /**
          @return the node id of the share
        */
        QString nodeId() const;

        /**
          @return the volume id of the share
        */
        QString volumeId() const;

        /**
          The free space left on the share. This is especially interesting
          information for inbound shares.

          While outbound shares will generally have the same free space as your
          Ubuntu One subscription in general, the space of inbound shares is
          dependent on the space of the sharing party.
          For a writable inbound share this means that size restrictions are
          completely different from usual space avilable for the rest of the
          user's Ubuntu One account/subscription.

          @return the free space available on the share (in bytes)
        */
        uint freeBytes() const;

        /**
          Returns the local path under which the share data may be found.

          @return local path of share
        */
        QString path() const;

        /**
          @return @c true if the share was accepted
          @return @c false if the share was not accepted
        */
        bool isAccepted() const;

        /**
          @return type of the share
        */
        Share::Type type() const;

        /**
          Helper function taking an input string and returning the appropriate
          Access type for the string.

          @param access the string for which to return the Access representation,
                 if the string is unknown the returned value is unspecified

          @return Access associated with string
        */
        static Share::Access stringToAccess(const QString &access);

        /**
          Helper function taking an input Access and returning the appropriate
          string representation for the Access.

          @param access the Access type for which to return the string
                 representation

          @return string representation of input Access
        */
        static QString accessToString(Share::Access access);

        /**
          Helper function taking an input string and returning the appropriate
          Type type for the string.

          @param type the string for which to return the Type representation,
                 if the string is unknown the returned value is unspecified

          @return Type associated with string
        */
        static Share::Type stringToType(const QString &type);

        /**
          Helper function taking an input Type and returning the appropriate
          string representation for the Type.

          @param type the Type type for which to return the string
                 representation

          @return string representation of input Type
        */
        static QString typeToString(Share::Type type);

        /**
          Registers the Share class with the Qt DBus type system.
        */
        static void registerMetaType();

        friend const QDBusArgument &operator>>(const QDBusArgument &argument,
                                               Share &share);
        friend QDBusArgument &operator<<(QDBusArgument &argument,
                                         const Share &share);

    protected:
        SharePrivate *const d_ptr;

    private:
        Q_DECLARE_PRIVATE(Share)
        };

    typedef QList<Share> ShareList;
} // namespace UbuntuOne

#endif // SHARE_H
