/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Status.h"

#include "QtDBus/QDBusMetaType"

#include "Types.h"

namespace UbuntuOne
{

    class StatusPrivate
    {
    public:
        StatusPrivate() :
            connected(false),
            online(false),
            error(false) {
        }

        QString name;
        QString queues;
        QString connection;
        QString description;
        bool connected;
        bool online;
        bool error;

        Status::State state;
        Status::ErrorState errorState;
    };

    Status::Status(QObject *parent) :
        QObject(parent),
        d_ptr(new StatusPrivate)
    {
    }

    Status::Status(const Status &other) :
        QObject(other.parent()),
        d_ptr(new StatusPrivate)
    {
        Q_D(Status);

        d->name = other.d_ptr->name;
        d->queues = other.d_ptr->queues;
        d->connection = other.d_ptr->connection;
        d->description = other.d_ptr->description;
        d->connected = other.d_ptr->connected;
        d->online = other.d_ptr->online;
        d->error = other.d_ptr->error;
        d->state = other.d_ptr->state;
        d->errorState = other.d_ptr->errorState;
    }

    Status::~Status()
    {
        delete d_ptr;
    }

    void Status::registerMetaType()
    {
        qRegisterMetaType<Status>("Status");
        qDBusRegisterMetaType<Status>();
    }

    Status::State Status::state() const
    {
        Q_D(const Status);

        return d->state;
    }

    Status::ErrorState Status::errorState() const
    {
        Q_D(const Status);

        return d->errorState;
    }

    QString Status::name() const
    {
        Q_D(const Status);

        return d->name;
    }

    bool Status::isConnected() const
    {
        Q_D(const Status);

        return d->connected;
    }

    const QDBusArgument &operator>>(const QDBusArgument &argument,
                                    UbuntuOne::Status &status)
    {
        QStringHash myhash;
        argument >> myhash;

        StatusPrivate *const d = status.d_func();
        d->name = myhash["name"];
        d->queues = myhash["queues"];
        d->connection = myhash["connection"];
        d->description = myhash["description"];
        d->connected = myhash["is_connected"] == QString("True") ? true : false;
        d->online = myhash["is_online"] == QString("True") ? true : false;
        d->error = myhash["is_error"] == QString("True") ? true : false;

        QString state = status.name();
        Status::State tmp_state = Status::Disconnected;
        Status::ErrorState tmp_errorState = Status::UnkownError;

        if (state == "OFFLINE" || state.startsWith(QLatin1String("INIT")) || state.startsWith(QLatin1String("READY"))) {
            tmp_state = Status::Disconnected;
        } else if (state == "CAPBILITIES_MISMATCH") {
            tmp_state = Status::Error;
            tmp_errorState = Status::CapabilitiesMismatch;
        } else if (state == "AUTH_FAILED") {
            tmp_state = Status::Error;
            tmp_errorState = Status::AuthFailed;
        } else if (state == "UNKNOWN_ERROR") {
            tmp_state = Status::Error;
            tmp_errorState = Status::UnkownError;
        } else if (state.startsWith(QLatin1String("CONNECTING")) || state.startsWith(QLatin1String("START_CONNECTING")) ||
                   state.startsWith(QLatin1String("AUTHENTICATING")) || state.startsWith(QLatin1String("CONNECTED")) ||
                   state.startsWith(QLatin1String("START_CONNECTED"))) {
            tmp_state = Status::Connecting;
        } else {
            tmp_state = Status::Idle;
        }

        d->state = tmp_state;
        d->errorState = tmp_errorState;

        return argument;
    }

    QDBusArgument &operator<<(QDBusArgument &argument,
                              const UbuntuOne::Status &status)
    {
        Q_UNUSED(status);
        // TODO: Implement proper when needed.
        argument.beginMap(QVariant::String, QVariant::String);
        argument.endMap();
        return argument;
    }

} // namespace UbuntuOne
