/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_KDE_SYNCDAEMONSTATUS_H
#define U1_KDE_SYNCDAEMONSTATUS_H

#include <QtCore/QObject>
#include <QtDBus/QDBusArgument>

#include "Export.h"

namespace UbuntuOne
{
    class StatusPrivate;

    /**
      A Ubuntu One status (only used within SyncDaemon itself). A status
      is a container for all vita information regarding Ubuntu One, or in
      particular about the SyncDaemon. Every time one of the status items chanes
      a new status object ought to be emitted, rendering the previous one
      useless.
    */
    class U1_KDE_EXPORT Status : public QObject
    {
        Q_OBJECT

    public:
        /**
          The various states the Ubuntu One SyncDaemon can be in.
        */
        enum State {
            Error = -1, /**< An error has occured, also see ErrorState enum. */
            Disconnected = 0, /**<  The SyncDaemon is disconnected from the cloud. */
            Idle = 1, /**< The SyncDaemon is connected to the cloud. */
            Connecting = 2 /**< The SyncDaemon is trying to connect to the cloud. */
        };

        /**
          When the SyncDaemon is in State::Error one of the following ErrorStates
          is available.
        */
        enum ErrorState {
            UnkownError = 0, /**< Unknown error of any sort (e.g. exception where none was expected). */
            AuthFailed = 1, /**< Authentication using Ubuntu SSO failed for some reason. */
            CapabilitiesMismatch = 2 /**< The local client needs an update. */
        };

        Status(QObject *parent = 0);
        Status(const Status &other);
        virtual ~Status();

        /**
          @return the state this status object represents
        */
        State state() const;

        /**
          The error state of the status object. Please note that this will
          return and unspecified value if the state() is not State::Error.

          @return the errorstate this status object represents
        */
        ErrorState errorState() const;

        /**
          @return the name
        */
        QString name() const;

        /**
          @return @c true if the SyncDaemon is connected
          @return @c false if the SyncDaemon is not connected
        */
        bool isConnected() const;

        /**
          Registers the Status class with the Qt DBus type system.
        */
        static void registerMetaType();

        friend const QDBusArgument &operator>>(const QDBusArgument &argument,
                                               UbuntuOne::Status &status);

        friend QDBusArgument &operator<<(QDBusArgument &argument,
                                         const UbuntuOne::Status &status);

    protected:
        StatusPrivate *const d_ptr;

    private:
        Q_DECLARE_PRIVATE(Status);
    };

} // namespace UbuntuOne

Q_DECLARE_METATYPE(UbuntuOne::Status);

#endif // SYNCDAEMONSTATUS_H
