/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_KDE_SYNCDAEMON_H
#define U1_KDE_SYNCDAEMON_H

#include <QtCore/QObject>

#include "Export.h"
#include "Types.h"

namespace UbuntuOne
{
    class Status;
    class SyncDaemonPrivate;

    /**
      Main interface to the Ubuntu One Sync Daemon.

      Really this only provides access to core functionality and status queries
      since those are the most essential parts of the Sync Daemon's actions.
    */
    class U1_KDE_EXPORT SyncDaemon : public QObject
    {
        Q_OBJECT

    public:
        SyncDaemon(QObject *parent = 0);
        ~SyncDaemon();

        /**
          Root directory of Ubuntu One.

          @return path of the directory
        */
        QString rootDir() const;

    Q_SIGNALS:
        /**
          This signal is emitted when the SyncDaemon gets disconnected from the cloud.
        */
        void disconnected();

        /**
          This signal is emitted when the SyncDaemon goes into idle state.
        */
        void idle();

        /**
          This signal is emitted when the SyncDaemon tries to connect to the cloud.
        */
        void connecting();

        /**
          This signal is emitted when an unkown error occurred.
        */
        void unkownErrorOccurred();

        /**
          This signal is emitted when authentication using the Ubuntu Single
          Sign On service failed.
        */
        void authFailed();

        /**
          This signal is emitted when there is a capabilities mismatch between
          cloud and the local SyncDaemon. This usually can be fixed by upgrading
          the client.
        */
        void capabilitiesMismatch();

        /**
          This signal is emitted when the transfer queue changed.
        */
        void queueUpdated(int queueCount);

        /**
          This signals is emitted when new files are being uploaded.

          @param files the files (or file) that are being uploaded
          @param queueCount amount of items left in the queue
        */
        void uploading(const QStringList &files, int queueCount);

        /**
          This signals is emitted when new files are being downloaded.

          @param files the files (or file) that are being downloaded
          @param queueCount amount of items left in the queue
        */
        void downloading(const QStringList &files, int queueCount);

    public Q_SLOTS:
        /**
          Connect to cloud.
        */
        void connectDaemon() const;

        /**
          Disconnect from cloud.
        */
        void disconnectDaemon() const;

        /**
          Restart daemon.
        */
        void restartDaemon() const;

        /**
          Called when the SyncDaemon status changed.

          @param status Status object containing current status data.
        */
        void statusChanged(const UbuntuOne::Status &status);

        /**
          Update the current status information.
        */
        void updateStatus();

    private Q_SLOTS:
        void receivedCredentials(const QString &realm, const QString &consumerKey);
        void contentQueueChanged(const QStringHashHash &queue);
        void uploadStarted(const QString &path);
        void uploadFinished(const QString &path, const QStringHash &info);
        void downloadStarted(const QString &path);
        void downloadFinished(const QString &path, const QStringHash &info);

        void debugDBusMessage(const QDBusMessage &msg);

    protected:
        SyncDaemonPrivate *const d_ptr;

    private:
        Q_DISABLE_COPY(SyncDaemon)
        Q_DECLARE_PRIVATE(SyncDaemon)
    };

} // namespace UbuntuOne

#endif // SYNCDAEMON_H
