/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "SyncDaemonFolders.h"

#include "SyncDaemonDBus.h"

namespace UbuntuOne {

    class SyncDaemonFoldersPrivate
    {
    public:
        ComUbuntuoneSyncDaemonFoldersInterface *folders;
    };

    SyncDaemonFolders::SyncDaemonFolders(QObject *parent) :
            QObject(parent),
            d_ptr(new SyncDaemonFoldersPrivate)
    {
        qRegisterMetaType<QStringHash>("QStringHash");
        qDBusRegisterMetaType<QStringHash>();
        qRegisterMetaType<QStringHashList>("QStringHashList");
        qDBusRegisterMetaType<QStringHashList>();
        Folder::registerMetaType();

        Q_D(SyncDaemonFolders);
        d->folders =
                new ComUbuntuoneSyncDaemonFoldersInterface ("com.ubuntuone.SyncDaemon",
                                                            "/folders", QDBusConnection::sessionBus(), 0);

        connect(d->folders, SIGNAL(FolderCreated(UbuntuOne::Folder)),
                this, SIGNAL(created(UbuntuOne::Folder)));
        connect(d->folders, SIGNAL(FolderCreateError(UbuntuOne::Folder,QString)),
                this, SIGNAL(creationError(UbuntuOne::Folder,QString)));
        connect(d->folders, SIGNAL(FolderDeleted(UbuntuOne::Folder)),
                this, SIGNAL(deleted(UbuntuOne::Folder)));
        connect(d->folders, SIGNAL(FolderSubscribed(UbuntuOne::Folder)),
                this, SIGNAL(subscribed(UbuntuOne::Folder)));
        connect(d->folders, SIGNAL(FolderSubscribeError(UbuntuOne::Folder,QString)),
                this, SIGNAL(subscribeError(UbuntuOne::Folder,QString)));
        connect(d->folders, SIGNAL(FolderUnSubscribed(UbuntuOne::Folder)),
                this, SIGNAL(unsubscribed(UbuntuOne::Folder)));
        connect(d->folders, SIGNAL(FolderUnSubscribeError(UbuntuOne::Folder,QString)),
                this, SIGNAL(unsubscribeError(UbuntuOne::Folder,QString)));
    }

    SyncDaemonFolders::~SyncDaemonFolders()
    {
        delete d_ptr;
    }

    Folder SyncDaemonFolders::folder(const QString path) const
    {
        Q_D(const SyncDaemonFolders);
        return d->folders->get_info(path);
    }

    void SyncDaemonFolders::add(const QString &path)
    {
        Q_D(SyncDaemonFolders);
        d->folders->create(path);
    }

    void SyncDaemonFolders::remove(const QString &path)
    {
        Q_D(SyncDaemonFolders);
        QList<QVariant> argumentList;
        argumentList << qVariantFromValue(folder(path).volumeId());
        d->folders->asyncCallWithArgumentList(QLatin1String("delete"), argumentList);
    }

    FolderList SyncDaemonFolders::folders() const
    {
        Q_D(const SyncDaemonFolders);
        return d->folders->get_folders();
    }

    QStringList SyncDaemonFolders::folderList() const
    {
        Q_D(const SyncDaemonFolders);
        const FolderList list = d->folders->get_folders();
        QStringList out;
        foreach(const Folder &f, list) {
            out << f.path();
        }
        return out;
    }

    void SyncDaemonFolders::changeFolders(const QStringList &folders)
    {
        const QStringList existing = folderList();
        // Remove missing ones.
        foreach (const QString &s, existing) {
            if (!folders.contains(s)) {
                remove(s);
            }
        }
        // Add new folders.
        foreach (const QString &s, folders) {
            if (!existing.contains(s)) {
                add(s);
            }
        }
#warning FIXME removal of former subdirs must be carried out before adding the new parent (block?)
    }

    void SyncDaemonFolders::refreshFolderList() const
    {
        Q_D(const SyncDaemonFolders);
        d->folders->refresh_volumes();
    }

    void SyncDaemonFolders::subscribe(const QString &folderId)
    {
        Q_D(SyncDaemonFolders);
        d->folders->subscribe(folderId);
    }

    void SyncDaemonFolders::unsubscribe(const QString &folderId)
    {
        Q_D(SyncDaemonFolders);
        d->folders->unsubscribe(folderId);
    }

} // namespace UbuntuOne
