/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_KDE_SYNCDAEMONFOLDERS_H
#define U1_KDE_SYNCDAEMONFOLDERS_H

#include <QtCore/QObject>

#include "Export.h"
#include "Folder.h"

namespace UbuntuOne {
    class SyncDaemonFoldersPrivate;

    /**
      SyncDaemon interface to manage Ubuntu One folders.
    */
    class U1_KDE_EXPORT SyncDaemonFolders : public QObject
    {
        Q_OBJECT
    public:
        explicit SyncDaemonFolders(QObject *parent = 0);
        ~SyncDaemonFolders();

        /**
          Get folder information about a specific path.

          @param path path to get information for.
        */
        Folder folder(const QString path) const;

        /**
          Adds a new folder to be synced with Ubuntu One.

          @path path of the folder to sync.
        */
        void add(const QString &path);

        /**
          Remove a folder from syncing with Ubuntu One.

          @path path of the folder to stop syncing.
        */
        void remove(const QString &path);

        /**
          Get a list of all folders.

          @return a list of folders.
        */
        FolderList folders() const;

        /**
          Get a list of all folders.

          @return a list of paths of folders.
        */
        QStringList folderList() const;

        /**
          Change existing folders by evaluating an input set of folder paths.
          This function is a convenience function to easily add and/or remove
          folders to/from the cloud. The input folder list ought to be similar
          to the one received from folderList.
          In particular this function will compare the two lists and remove
          missing folders from Ubuntu One or add new ones that are not yet
          present in the folderList() output to be synchronized with the cloud.

          @param folders a list of folders that compared to the output of
                 folderList() will trigger appropriate actions so that folders
                 will eventually become the new output of folderList()
        */
        void changeFolders(const QStringList &folders);

    Q_SIGNALS:
        /**
          This signal is emitted whenever a new Folder was successfully created.

          @param folder the Folder that was created.
        */
        void created(const UbuntuOne::Folder &folder);

        /**
          This signal is emitted when creation of a new Folder failed.

          @param folder the Folder that ought to have been created.
          @param error description of the error.
        */
        void creationError(const UbuntuOne::Folder &folder, const QString &error);

        /**
          This signal is emitted whenever a Folder was successfully deleted.

          @param folder the Folder that was deleted.
        */
        void deleted(const UbuntuOne::Folder &folder);

        /**
          This signal is emitted when deletion of a Folder failed.

          @param folder the Folder that ought to have been deleted.
          @param error description of the error.
        */
        void deleteError(const UbuntuOne::Folder &folder, const QString &error);

        /**
          This signal is emitted whenever the user successfully subscribed to
          a Folder.

          @param folder the Folder that was subscribed.
        */
        void subscribed(const UbuntuOne::Folder &folder);

        /**
          This signal is emitted when subscription of a Folder failed.

          @param folder the Folder that ought to have been subscribed.
          @param error description of the error.
        */
        void subscribeError(const UbuntuOne::Folder &folder, const QString &error);

        /**
          This signal is emitted whenever the user successfully unsubscribed
          a Folder.

          @param folder the Folder that was unsubscribed.
        */
        void unsubscribed(const UbuntuOne::Folder &folder);

        /**
          This signal is emitted when unsubscription of a Folder failed.

          @param folder the Folder that ought to have been unsubscribed.
          @param error description of the error.
        */
        void unsubscribeError(const UbuntuOne::Folder &folder, const QString &error);

    public Q_SLOTS:
        /**
          Refresh the list of folders from the Ubuntu One servers.
        */
        void refreshFolderList() const;

        /**
          Subscribe to a folder from Ubuntu One.

          @param folderId folder id to subscribe to.
        */
        void subscribe(const QString &folderId);

        /**
          Unsubscribe from a folder from Ubuntu One.

          @param folderId folder id to unsubscribe from.
        */
        void unsubscribe(const QString &folderId);

    protected:
        SyncDaemonFoldersPrivate *const d_ptr;

    private:
        Q_DISABLE_COPY(SyncDaemonFolders)
        Q_DECLARE_PRIVATE(SyncDaemonFolders)
    };

} // namespace UbuntuOne

#endif // SYNCDAEMONFOLDERS_H
