/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef U1_KDE_SYNCDAEMONSHARES_H
#define U1_KDE_SYNCDAEMONSHARES_H

#include <QtCore/QObject>

#include "Export.h"
#include "Share.h"

namespace UbuntuOne {
    class SyncDaemonSharesPrivate;

    /**
      SyncDaemon interface to manage Ubuntu One shares (inbound and outbound).
    */
    class U1_KDE_EXPORT SyncDaemonShares : public QObject
    {
        Q_OBJECT

    public:
        /**
          Constructor.
        */
        explicit SyncDaemonShares(QObject *parent = 0);

        /**
          Destructor.
        */
        ~SyncDaemonShares();

        /**
          Create new share.

          @param path The path to share.
          @param contacts Email addresses to share with.
          @param name The name of the share.
          @param accessLevel the ShareAccess of the share (read-only...).
        */
        void share(const QString &path, const QStringList &contacts,
                   const QString &name, Share::Access accessLevel) const;

        /**
          Get a list of all shares (inbound).

          @return a list of (inbound) shares.
        */
        ShareList shares() const;

        /**
          Get a list of all shared folders.

          @return a list of paths of shares (inbound) folders.
        */
        QStringList sharesFolders() const;

        /**
          Get a list of all shared folders (outbound).

          @return a list of (outbound) shares.
        */
        ShareList shared() const;

        /**
          Get a list of all shared folders.

          @return a list of paths of shared (outbound) folders.
        */
        QStringList sharedFolders() const;

    Q_SIGNALS:
        /**
          This signal is emitted whenever a new (inbound) share arrived.

          @param share the share information for the new share.
        */
        void newShare(const UbuntuOne::Share &share);

        /**
          This signal is emitted when a new share answer response arrived.

          @param share the share answer.
        */
        void shareAnswerRepsone(const UbuntuOne::Share &share);

        /**
          This signal is emitted whenever a share changed.

          @param share the share information for the changed share.
        */
        void shareChanged(const UbuntuOne::Share &share);

        /**
          This signal is emitted whenever a new share was created.

          @param share the share information for the new share.
        */
        void shareCreated(const UbuntuOne::Share &share);

        /**
          This signal is emitted when an error was encountered while trying to
          create a new share.

          @param share the share information.
          @param error the error string describing what went wrong.
        */
        void shareCreateError(const UbuntuOne::Share &share,
                              const QString &error);

        /**
          This signal is emitted whenever a share was removed.

          @param share the share information for the removed share.
        */
        void shareRemoved(const UbuntuOne::Share &share);

        /**
          This signal is emitted when an error was encountered while trying to
          delete an existing share.

          @param share the share information.
          @param error the error string describing what went wrong.
        */
        void shareRemoveError(const UbuntuOne::Share &share,
                              const QString &error);

    public Q_SLOTS:
        /**
          Accepts a share.

          @param shareId the id of the share to accept.
        */
        void accept(const QString &shareId);

        /**
          Reject a share.

          @param shareId the id of the share to reject.
        */
        void reject(const QString &shareId);

        /**
          Remove a share.

          @param shareId the id of the share to remove.
        */
        void remove(const QString &shareId);

        /**
          Refresh the list of shares from the Ubuntu One servers.
        */
        void refreshShareList() const;

    protected:
        SyncDaemonSharesPrivate *const d_ptr;

    private:
        Q_DISABLE_COPY(SyncDaemonShares)
        Q_DECLARE_PRIVATE(SyncDaemonShares)
    };

} // namespace UbuntuOne

#endif // SYNCDAEMONSHARES_H
