/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ContactSelector.h"

#include <Akonadi/ChangeRecorder>
#include <akonadi/contact/contactstreemodel.h>
#include <Akonadi/EntityDisplayAttribute>
#include <Akonadi/EntityTreeView>
#include <Akonadi/ItemFetchScope>
#include <Akonadi/Session>
#include <KABC/Addressee>
#include <KABC/ContactGroup>
#include <KFileTreeView>
#include <KLineEdit>
#include <KLocalizedString>

#include <QtCore/QDebug>
#include <QtGui/QHeaderView>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QSortFilterProxyModel>

class FilterModel : public QSortFilterProxyModel
{
public:
    FilterModel(QObject *parent = 0) : QSortFilterProxyModel(parent) {}

    virtual bool filterAcceptsRow(int sourceRow,
                                  const QModelIndex &sourceParent) const
    {
        if (!sourceParent.isValid()) { // Parent node == Akonadi Resource -> always match
            return true;
        }
        return QSortFilterProxyModel::filterAcceptsRow(sourceRow, sourceParent);
    }
};

namespace UbuntuOne {
    ContactSelector::ContactSelector(QWidget *parent) :
            QWidget(parent)
    {
        QVBoxLayout *layout = new QVBoxLayout(this);

        KLineEdit *filterLine = new KLineEdit(this);
        filterLine->setClickMessage(i18nc("@label:textbox Search contacts in list",
                                          "Search"));
        layout->addWidget(filterLine);

        m_session = new Akonadi::Session("ubuntuone-share");

        Akonadi::ItemFetchScope scope;
        scope.fetchFullPayload(true);
        scope.fetchAttribute<Akonadi::EntityDisplayAttribute>();

        m_changeRecorder = new Akonadi::ChangeRecorder;
        m_changeRecorder->setSession(m_session);
        m_changeRecorder->fetchCollection(true);
        m_changeRecorder->setItemFetchScope(scope);
        m_changeRecorder->setCollectionMonitored(Akonadi::Collection::root());
        m_changeRecorder->setMimeTypeMonitored(KABC::Addressee::mimeType(), true);
        m_changeRecorder->setMimeTypeMonitored(KABC::ContactGroup::mimeType(), true);

        m_model = new Akonadi::ContactsTreeModel(m_changeRecorder);
        Akonadi::ContactsTreeModel::Columns columns;
        columns << Akonadi::ContactsTreeModel::FullName;
        columns << Akonadi::ContactsTreeModel::PreferredEmail;
        m_model->setColumns(columns);

        FilterModel *filterModel = new FilterModel(this);
        filterModel->setSourceModel(m_model);
        filterModel->setDynamicSortFilter(true);
        filterModel->setFilterCaseSensitivity(Qt::CaseInsensitive);

        m_view = new Akonadi::EntityTreeView;
        m_view->setModel(filterModel);
        m_view->header()->setResizeMode(0, QHeaderView::Stretch);
        m_view->setSortingEnabled(true);
        m_view->setSelectionMode(QAbstractItemView::ExtendedSelection);
        layout->addWidget(m_view);

        connect(filterLine, SIGNAL(textChanged(QString)),
                filterModel, SLOT(setFilterFixedString(QString)));
        connect(filterLine, SIGNAL(textChanged(QString)),
                m_view, SLOT(expandAll()));
        connect(m_view->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
                this, SIGNAL(itemSelected()));
    }

    ContactSelector::~ContactSelector()
    {
        delete m_view;
        delete m_model;
        delete m_session;
        delete m_changeRecorder;
    }

    QStringList ContactSelector::addresses() const
    {
        QStringList list;
        int indexOfMail = m_model->columns().indexOf(Akonadi::ContactsTreeModel::PreferredEmail);
        QModelIndexList indexList = m_view->selectionModel()->selectedRows(indexOfMail);
        foreach(const QModelIndex &index, indexList) {
            QString mail = index.data().toString();
            if (!mail.isEmpty()) {
                list << mail;
            }
        }
        qDebug() << list;
        return list;
    }
} // namespace UbuntuOne
