/*
  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ShareDialog.h"

#include <QtCore/QCoreApplication>
#include <QtGui/QLineEdit>

#include <KDirSelectDialog>
#include <KLineEdit>
#include <KLocalizedString>
#include <KMessageBox>
#include <KPushButton>

#include "ContactSelector.h"
#include "SharePropertiesWidget.h"
#include "SyncDaemon.h"
#include "SyncDaemonFolders.h"
#include "SyncDaemonShares.h"

namespace UbuntuOne {
    ShareDialog::ShareDialog() :
            KAssistantDialog(),
            m_folders(0),
            m_dirSelect(0),
            m_contactSelector(new ContactSelector(this)),
            m_shareProperties(new SharePropertiesWidget(this)),
            m_newFolder(QString()),
            m_waiting(false)
    {
        showButton(KDialog::Help, false);

        SyncDaemon s;
        m_dirSelect = new KDirSelectDialog(KUrl(s.rootDir()), true, this);
        m_dirSelect->setButtons(KDialog::None);

        setButtonGuiItem(KDialog::Try, KGuiItem(i18nc("@action:button","New Folder..."),
                                                    "folder-new"));

        addPage(m_dirSelect, i18nc("@title:tab", "Select a directory to share via Ubuntu One"));
        addPage(m_contactSelector, i18nc("@title:tab", "Select contacts to share with"));
        addPage(m_shareProperties, i18nc("@title:tab", "Set the properties for the new share"));

        setButtonText(KDialog::User1, i18nc("@action:button", "Share"));

        connect(m_contactSelector, SIGNAL(itemSelected()),
                this, SLOT(updateNextButton()));
        connect(m_shareProperties->nameLine(), SIGNAL(textChanged(QString)),
                this, SLOT(updateShareButton(QString)));
        connect(this, SIGNAL(tryClicked()), m_dirSelect, SLOT(slotNewFolder()));
        connect(this, SIGNAL(user1Clicked()), this, SLOT(share()));
    }

    ShareDialog::~ShareDialog()
    {
    }

    void ShareDialog::next()
    {
        if (!canProceed()) {
            return;
        }
        KAssistantDialog::next();
        updateShareButton(m_shareProperties->nameLine()->text());
        updateNextButton();
    }

    void ShareDialog::share()
    {
        if (!m_newFolder.isEmpty()) {
            m_waiting = true;
            connect(m_folders, SIGNAL(created(UbuntuOne::Folder)),
                    this, SLOT(unsetWaiting()));
            m_folders->add(m_newFolder);
        }

#warning TODO <Riddell> not very elegant of course
        while (m_waiting) {
            QCoreApplication::instance()->processEvents();
        }

        QString path = m_dirSelect->url().path();
        QStringList contacts = m_contactSelector->addresses();
        QString name = m_shareProperties->name();
        Share::Access access = m_shareProperties->access();
        SyncDaemonShares s;
        s.share(path, contacts, name, access);
    }

    void ShareDialog::updateNextButton()
    {
        if (currentPage()->widget() != m_contactSelector) {
            return; // Leave unless we are on the show stopping page.
        }

        if (m_contactSelector->addresses().isEmpty()) {
            button(KDialog::User2)->setEnabled(false);
        } else {
            button(KDialog::User2)->setEnabled(true);
        }
    }

    void ShareDialog::updateShareButton(const QString &string)
    {
        if (!string.isEmpty()) {
            button(KDialog::User1)->setEnabled(true);
        } else {
            button(KDialog::User1)->setEnabled(false);
        }
    }

    void ShareDialog::unsetWaiting()
    {
        m_waiting = false;
    }

    bool ShareDialog::canProceed()
    {
        if (currentPage()->widget() == m_dirSelect) {
            if (m_folders == 0) {
                m_folders = new SyncDaemonFolders(this);
            }
            QStringList folders = m_folders->folderList();
            // Add root directory to the list (not covered by folderList).
            folders.append(m_dirSelect->startDir().path());
            QString selectedFolder = m_dirSelect->url().path();

            bool match = false;
            foreach(const QString &folder, folders) {
                if (selectedFolder.contains(folder)) {
                    match = true;
                }
            }

            if (!match) {
                if (!m_newFolder.isEmpty() && m_newFolder == selectedFolder) {
                    // Already marked as to-be-synced before, no need to ask again.
                    return true;
                }
                KGuiItem yes = KStandardGuiItem::yes();
                yes.setText(i18nc("@action:button", "Synchronize"));
                KGuiItem no = KStandardGuiItem::no();
                no.setText(i18nc("@action:button", "Do Not Synchronize"));
                int ret = KMessageBox::questionYesNo(this,
                     i18nc("@info", "The folder you selected is currently not synchronized with the Ubuntu One cloud.<br/><br/>"
                           "Do you want to synchronize it so you can share it?<br/><br/>"
                           "(Please note that this can take quite some time, so it might be a better idea to do this in the Ubuntu One settings.)"),
                     i18nc("@title:window", "Folder Can't be Shared"),
                     yes, no);
                if (ret == KMessageBox::Yes) {
                    m_newFolder.clear();
                    m_newFolder = selectedFolder;
                } else {
                    return false;
                }
            }
        }
        return true;
    }

} // namespace UbuntuOne
