/*
  Copyright © 2009-2010 Harald Sitter <apachelogger@ubuntu.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License or (at your option) version 3 or any later version
  accepted by the membership of KDE e.V. (or its successor approved
  by the membership of KDE e.V.), which shall act as a proxy
  defined in Section 14 of version 3 of the license.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "StatusNotifier.h"

#include <QtGui/QProgressBar>

#include <KAboutData>
#include <KAction>
#include <KActionCollection>
#include <KCMultiDialog>
#include <KHelpMenu>
#include <KIcon>
#include <KLocalizedString>
#include <KNotification>
#include <KMenu>
#include <KMessageBox>
#include <KToolInvocation>
#include <KWebView>

#include "SyncDaemon.h"

namespace UbuntuOne
{
    StatusNotifier::StatusNotifier(QObject *parent) :
        KStatusNotifierItem(parent),
        m_working(false),
        m_view(0)
    {
        setCategory(ApplicationStatus);

        d = new SyncDaemon(this);

        KActionCollection *collection = actionCollection();
        KMenu *menu = contextMenu();

        QAction *quitAction = collection->action(KStandardAction::name(KStandardAction::Quit));
        quitAction->disconnect();
        connect(quitAction, SIGNAL(triggered()), this, SLOT(maybeQuit()));

        KAction *connectAction = new KAction(KIcon("network-connect"),
                                             i18nc("@action:button", "&Connect"), this);
        connect(connectAction, SIGNAL(triggered()), d, SLOT(connectDaemon()));
        connect(connectAction, SIGNAL(triggered()), this, SLOT(setConnecting()));
        connectAction->setVisible(false);
        collection->addAction("connect", connectAction);
        menu->addAction(collection->action("connect"));

        KAction *disconnectAction = new KAction(KIcon("network-disconnect"),
                                                i18nc("@action:button", "&Disconnect"), this);
        disconnectAction->setVisible(false);
        connect(disconnectAction, SIGNAL(triggered()), d, SLOT(disconnectDaemon()));
        collection->addAction("disconnect", disconnectAction);
        menu->addAction(collection->action("disconnect"));

        KAction *restartAction = new KAction(KIcon("view-refresh"),
                                             i18nc("@action:button Restarts the Ubuntu One daemon", "&Restart"), this);
        connect(restartAction, SIGNAL(triggered()), d, SLOT(restartDaemon()));
        collection->addAction("restart", restartAction);
        menu->addAction(collection->action("restart"));

        KAction *share = new KAction(KIcon("user-group-new"),
                                     i18nc("@action:button Opens the share helper application", "&Share"), this);
        connect(share, SIGNAL(triggered()), this, SLOT(launchShare()));
        menu->addAction(share);

        KAction *configAction = new KAction(KIcon("configure"),
                                            i18nc("@action:button", "Con&figure"), this);
        connect(configAction, SIGNAL(triggered()), this, SLOT(launchConfig()));
        collection->addAction("config", configAction);
        menu->addAction(collection->action("config"));

        // Add a cleaned up KHelpMenu. Remove handbook (since we do not have one)
        // and whatisthis (does not make much sense for a try-only app).
        KHelpMenu *kHelpMenu = new KHelpMenu(contextMenu(), KGlobal::mainComponent().aboutData());
        QMenu *helpMenu = kHelpMenu->menu();
        QAction *handbookAction = kHelpMenu->action(KHelpMenu::menuHelpContents);
        helpMenu->removeAction(handbookAction);
        QAction *whatIsThisAction = kHelpMenu->action(KHelpMenu::menuWhatsThis);
        helpMenu->removeAction(whatIsThisAction);
        menu->addMenu(helpMenu);

        setDisconnected();

        connect(d, SIGNAL(idle()), this, SLOT(setIdle()));
        connect(d, SIGNAL(disconnected()), this, SLOT(setDisconnected()));
        connect(d, SIGNAL(connecting()), this, SLOT(setConnecting()));
        connect(d, SIGNAL(authFailed()), this, SLOT(setAuthFailed()));
        connect(d, SIGNAL(capabilitiesMismatch()), this, SLOT(setCapabilityMismatch()));
        connect(d, SIGNAL(unkownErrorOccurred()), this, SLOT(setFatalError()));

        connect(d, SIGNAL(queueUpdated(int)),
                this, SLOT(updateState(int)));
        connect(d, SIGNAL(uploading(QStringList,int)),
                this, SLOT(setUploadStatus(QStringList,int)));
        connect(d, SIGNAL(downloading(QStringList,int)),
                this, SLOT(setDownloadStatus(QStringList,int)));

        // To make it non-blocking...
        QTimer::singleShot(0, d, SLOT(updateStatus()));
    }

    StatusNotifier::~StatusNotifier()
    {
        if (m_view) {
            delete m_view;
        }
    }

    void StatusNotifier::activate(const QPoint &pos)
    {
        if (m_view == 0) {
            m_view = new KWebView();
            m_view->setUrl(QUrl("https://one.ubuntu.com"));
            m_view->setWindowTitle(KGlobal::mainComponent().aboutData()->programName());
            setAssociatedWidget(m_view);
        }
        KStatusNotifierItem::activate(pos);
    }

    // Shamelessly borrowed from Klipper. All hail the Klipper!
    void StatusNotifier::maybeQuit()
    {
        int autoStart =
                KMessageBox::questionYesNoCancel(0,
                i18nc("@label", "Should Ubuntu One start automatically when you login?"),
                i18nc("@title:window", "Automatically Start Ubuntu One?"),
                KGuiItem(i18nc("@action:button", "Start")),
                KGuiItem(i18nc("@action:button", "Do Not Start")),
                KStandardGuiItem::cancel(),
                "StartAutomatically");

        KConfigGroup config(KGlobal::config(), "General");
        if (autoStart == KMessageBox::Yes) {
            config.writeEntry("Autostart", true);
        } else if (autoStart == KMessageBox::No) {
            config.writeEntry("Autostart", false);
        } else { // cancel chosen don't quit
            return;
        }
        config.sync();

        qApp->quit();
    }

    void StatusNotifier::setAuthFailed()
    {
        setIconByName("ubuntuone-client-error");
        setToolTip(iconName(),
                   i18nc("@info:status", "Authentication failed"),
                   i18nc("@info:tooltip", "The authentication was not successful."));
        setStatus(NeedsAttention);
    }

    void StatusNotifier::setConnecting()
    {
        setIconByName("ubuntuone-client-idle");
        setToolTip(iconName(),
                   i18nc("@info:status", "Connecting to the cloud"),
                   i18nc("@info:tooltip", "Trying to establish a connection."));
        setStatus(Active);
    }

    void StatusNotifier::setCapabilityMismatch()
    {
        KNotification::event("capabilitiesMismatch",
                             i18nc("@info",
                                   "There was a capabilities mismatch while attempting "
                                   "to connect to the Ubuntu One server. You may "
                                   "have installed a newer version of the client, for "
                                   "which the server does not yet provide support. "
                                   "A new version of the server should be accessible "
                                   "soon. Please be patient while we update."));

        setIconByName("ubuntuone-client-error");
        setToolTip(iconName(),
                   i18nc("@info:status", "Capabilities Mismatch"),
                   i18nc("@info:tooltip", "Capabilities mismatch with the cloud."));
        setStatus(NeedsAttention);
    }

    void StatusNotifier::setDisconnected()
    {
        actionCollection()->action("connect")->setVisible(true);
        actionCollection()->action("disconnect")->setVisible(false);

        setIconByName("ubuntuone-client-disconnected");
        setToolTip(iconName(),
                   i18nc("@info:status Ubuntu One daemon is disconnected from the server",
                         "Disconnected"),
                   i18nc("@info:tooltip", "Not connected to the cloud."));
        setStatus(Active);
    }

    void StatusNotifier::setFatalError()
    {
        KNotification::event("fatalError",
                             i18nc("@info",
                                   "There was a fatal error in Ubuntu One. "
                                   "This may be a bug in the software. "
                                   "Please click on the Ubuntu One icon "
                                   "in your panel to report a bug."));

        setIconByName("ubuntuone-client-error");
        setToolTip(iconName(),
                   i18nc("@info:status", "Fatal Error"),
                   i18nc("@info:tooltip", "Please report a bug."));
        setStatus(NeedsAttention);
    }

    void StatusNotifier::setIdle()
    {
        actionCollection()->action("connect")->setVisible(false);
        actionCollection()->action("disconnect")->setVisible(true);

        if (m_working) {
            return;
        }

        setIconByName("ubuntuone-client-idle");
        setToolTip(iconName(),
                   i18nc("@info:status Ubuntu One daemon is connected but idle", "Idle"),
                   i18nc("@info:tooltip", "Connected to the Cloud."));
        setStatus(Passive);
    }

    void StatusNotifier::updateState(int queueCount)
    {
        if (queueCount == 0) {
            m_working = false;
            return setIdle();
        }

        m_working = true;
    }

    void StatusNotifier::setUploadStatus(const QStringList &files, int queue)
    {
        m_working = true;
        setIconByName("ubuntuone-client-updating");
        setToolTip(iconName(),
                   i18ncp("@info:status", "Uploading file", "Uploading files",
                          queue),
                   i18ncp("@info:tooltip %1 is a list of files, %2 is the size of the queue",
                          "%1.", "%1 (%2 jobs left in queue).",
                          files.join(" "), queue));
        setStatus(Active);
    }

    void StatusNotifier::setDownloadStatus(const QStringList &files, int queue)
    {
        m_working = true;
        setIconByName("ubuntuone-client-updating");
        setToolTip(iconName(),
                   i18ncp("@info:status", "Downloading file", "Downloading files",
                          queue),
                   i18ncp("@info:tooltip %1 is a list of files, %2 is the size of the queue",
                          "%1.", "%1 (%2 jobs left in queue).",
                          files.join(" "), queue));
        setStatus(Active);
    }

    void StatusNotifier::launchConfig()
    {
        KCMultiDialog kcm;
        kcm.addModule("ubuntuone-general");
        kcm.addModule("ubuntuone-folders");
        kcm.addModule("ubuntuone-shares");
        kcm.addModule("ubuntuone-web");
        kcm.exec();
    }

    void StatusNotifier::launchShare()
    {
        KToolInvocation::startServiceByDesktopName("ubuntuone-share");
    }
}
